# Copyright (C) 2022 RozK
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from engine import mat4, mat4_projection, mat4_lookat, resolve_input, set_input_mat4

class _Inputs:
    __slots__ = 'projection', 'view'

    def __init__(self, shader, projection, view):
        self.projection = resolve_input(shader, projection)
        self.view = resolve_input(shader, view)

class Camera:
    __slots__ = 'projection', 'view'

    def __init__(self):
        self.projection = mat4()
        self.view = mat4()

    def set_projection(self, half_fov, ratio, near_z, far_z):
        mat4_projection(self.projection, half_fov, ratio, near_z, far_z)

    def set_view(self, origin, lookat):
        mat4_lookat(self.view, origin, lookat)

    def resolve_inputs(self, shader, projection = b'u_projection', view = b'u_view'):
        return _Inputs(shader, projection, view)

    def update_inputs(self, inputs):
        set_input_mat4(inputs.projection, self.projection)
        set_input_mat4(inputs.view, self.view)
